# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import struct
import time
import confluent_kafka

from lsst.alert.packet import SchemaRegistry

__all__ = ['EopError', 'AlertConsumer']

class AlertError(Exception):
    """Base class for exceptions in this module.
    """
    pass


class EopError(AlertError):
    """Exception raised when reaching end of partition.

    Parameters
    ----------
    msg : Kafka message
        The Kafka message result from consumer.poll().
    """
    def __init__(self, msg):
        message = 'topic:%s, partition:%d, status:end, ' \
                  'offset:%d, key:%s, time:%.3f\n' \
                  % (msg.topic(), msg.partition(),
                     msg.offset(), str(msg.key()), time.time())
        self.message = message

    def __str__(self):
        return self.message


class AlertConsumer(object):
    """Creates an alert stream Kafka consumer for a given topic.

    Parameters
    ----------
    topic : `str`
        Name of the topic to subscribe to.
    schema : `lsst.alert.packet.Schema`, optional
        If provided, this schema is always used to decode packets received.
        Otherwise, an appropriate schema is retrieved from the registry.
    **kwargs
        Keyword arguments for configuring confluent_kafka.Consumer().
    """

    def __init__(self, topic, schema=None, **kwargs):
        self.topic = topic
        self.kafka_kwargs = kwargs
        self.schema = schema
        self.schema_registry = SchemaRegistry.from_filesystem()

    def __enter__(self):
        self.consumer = confluent_kafka.Consumer(**self.kafka_kwargs)
        self.consumer.subscribe([self.topic])
        return self

    def __exit__(self, type, value, traceback):
        # FIXME should be properly handling exceptions here, but we aren't
        self.consumer.close()

    def poll(self, timeout=1e-3):
        """Polls Kafka broker to consume topic.

        Parameters
        ----------
        timeout : `float`
            Time to wait for a new message before returning.

        Returns
        -------
        schema : `lsst.alert.packet.Schema` or `None`
            Schema used to decode message. `None` if no message was received
            before the `timeout` was reached.
        message : `dict` or `None`
            Decoded message. `None` if no message was received before the
            `timeout` was reached.
        """
        msg = self.consumer.poll(timeout)

        if msg is not None:
            if msg.error():
                raise EopError(msg)
            else:
                return self.decode_message(msg)
        else:
            return None, None

    def decode_message(self, msg):
        """Unpack and decode a received message.

        Parameters
        ----------
        msg : Kafka message
            The Kafka message resulting from calling `AlertConsumer.poll`.

        Returns
        -------
        schema : `schema`
            Schema used to decode message.
        message : `dict`
            Decoded message.

        Notes
        -----
        If this `AlertConsumer` has an associated schema, that will be used
        for decoding; otherwise, it will attempt to retrieve one from the
        registry.
        """
        raw_bytes = msg.value()
        schema_hash = struct.unpack("!I", raw_bytes[1:5])[0]
        if not self.schema:
            schema = self.schema_registry.get_by_id(schema_hash)
        else:
            schema = self.schema
        return schema, schema.deserialize(raw_bytes[5:])



#!/usr/bin/env python
#
# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Alert stream filter deployer.
"""

from __future__ import print_function
import argparse
import sys
import os
import inspect
from lsst.alert.stream import alertConsumer, alertProducer
from lsst.alert.stream import filterBase
from lsst.alert.stream import filters
from lsst.alert.packet import SchemaRegistry


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('broker', type=str,
                        help='Hostname or IP and port of Kafka broker.')
    parser.add_argument('topic', type=str,
                        help='Name of Kafka topic to listen to.')
    parser.add_argument('filterNum', type=int,
                        help='Number of the filter in range '
                        '(1-100) to deploy.')
    parser.add_argument('--group', type=str,
                        help='Globally unique name of the consumer group. '
                        'Consumers in the same group will share messages '
                        '(i.e., only one consumer will receive a message, '
                        'as in a queue). Default is value of $HOSTNAME.')

    args = parser.parse_args()
    fnum = args.filterNum

    # Configure consumer connection to Kafka broker
    cconf = {'bootstrap.servers': args.broker,
             'default.topic.config': {'auto.offset.reset': 'smallest'}}
    if args.group:
        cconf['group.id'] = args.group
    else:
        cconf['group.id'] = os.environ['HOSTNAME']

    pconf = {'bootstrap.servers': args.broker}

    # Choose filter class to deploy from filters module
    filter_class = inspect.getmembers(filters, inspect.isclass)[fnum][1]

    # Name output stream using filter class name
    topic_name = filter_class.__name__

    prod = alertProducer.AlertProducer(topic_name, **pconf)
    exp = filterBase.StreamExporter(prod)
    apply_filter = filter_class(exp)

    # Start consumer and print alert stream
    with alertConsumer.AlertConsumer(args.topic, **cconf) as streamReader:

        while True:
            try:
                schema, msg = streamReader.poll()

                if msg is None:
                    continue
                else:
                    # Apply filter to each alert
                    apply_filter(schema, msg)

            except alertConsumer.EopError as e:
                # Write when reaching end of partition
                sys.stderr.write(e.message)
            except IndexError:
                sys.stderr.write('%% Data cannot be decoded\n')
            except UnicodeDecodeError:
                sys.stderr.write('%% Unexpected data format received\n')
            except KeyboardInterrupt:
                sys.stderr.write('%% Aborted by user\n')
                sys.exit()



#!/usr/bin/env python
#
# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Generates batches of alerts coming from a CCD given template alert
content.
"""

from __future__ import print_function
import argparse
import glob
import time
import asyncio
from lsst.alert.stream import alertProducer
from lsst.alert.packet import retrieve_alerts


@asyncio.coroutine
def delay(wait_sec, function, *args):
    """Sleep for a given time before calling a function.
    Parameters
    ----------
    wait_sec
        Time in seconds to sleep before calling `function`.
    function
        Function to return after sleeping.
    """
    yield from asyncio.sleep(wait_sec)
    return function(*args)


@asyncio.coroutine
def schedule_delays(eventloop, function, argslist, interval=39):
    """Schedule delayed calls of functions at a repeating interval.
    Parameters
    ----------
    eventloop
        Event loop returned by asyncio.get_event_loop().
    function
        Function to be scheduled.
    argslist
        List of inputs for function to loop over.
    interval
        Time in seconds between calls.
    """
    counter = 1
    for arg in argslist:
        wait_time = interval - (time.time() % interval)
        yield from asyncio.ensure_future(delay(wait_time, function, arg))
        print('visits finished: {} \t time: {}'.format(counter, time.time()))
        counter += 1
    eventloop.stop()


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('broker', type=str,
                        help='Hostname or IP and port of Kafka broker.')
    parser.add_argument('topic', type=str,
                        help='Name of Kafka topic stream to push to.')
    args = parser.parse_args()

    # Configure producer connection to Kafka broker
    conf = {'bootstrap.servers': args.broker}
    streamProducer = alertProducer.AlertProducer(args.topic, **conf)

    # Scan for avro files
    root = "./data"
    files = [f for f in glob.glob("/".join([root, "*.avro"]))]
    files.sort()

    def send_visit(f):
        print('visit:', f[15:20], '\ttime:', time.time())
        # Load alert contents
        with open(f, mode='rb') as file_data:
            # TODO replace Avro files with visits having better S/N cut
            # for now, limit to first 10,000 alerts (current have ~70,000)
            schema, alert_packets = retrieve_alerts(file_data)
            alert_count = 0
            for record in alert_packets:
                if alert_count < 10000:
                    streamProducer.send(schema, record)
                    alert_count += 1
                else:
                    break
        streamProducer.flush()

    loop = asyncio.get_event_loop()
    asyncio.ensure_future(schedule_delays(loop, send_visit, files))
    loop.run_forever()
    loop.close()


if __name__ == "__main__":
    main()



import unittest

from lsst.alert.stream import serialization


class TestSerialization(unittest.TestCase):
    def test_serialize_alert_round_trip(self):
        alert = mock_alert(1)
        serialized = serialization.serialize_alert(alert)
        deserialized = serialization.deserialize_alert(serialized)
        self.assertEqual(1, deserialized["alertId"])


def mock_alert(alert_id):
    """Generate a minimal mock alert. """
    return {
        "alertId": alert_id,
        "diaSource": {
            # Below are all the required fields. Set them to zero.
            "midPointTai": 0,
            "diaSourceId": 0,
            "ccdVisitId": 0,
            "filterName": "",
            "programId": 0,
            "ra": 0,
            "decl": 0,
            "x": 0,
            "y": 0,
            "apFlux": 0,
            "apFluxErr": 0,
            "snr": 0,
            "psFlux": 0,
            "psFluxErr": 0,
            "flags": 0,
        }
    }



import io
import struct

import fastavro
from lsst.alert.packet import Schema


_ConfluentWireFormatHeader = struct.Struct(">bi")

latest_schema = Schema.from_file().definition


def serialize_confluent_wire_header(schema_version):
    """Returns the byte prefix for Confluent Wire Format-style Kafka messages.

    Parameters
    ----------
    schema_version : `int`
        A version number which indicates the Confluent Schema Registry ID
        number of the Avro schema used to encode the message that follows this
        header.

    Returns
    -------
    header : `bytes`
        The 5-byte encoded message prefix.

    Notes
    -----
    The Confluent Wire Format is described more fully here:
    https://docs.confluent.io/current/schema-registry/serdes-develop/index.html#wire-format
    """
    return _ConfluentWireFormatHeader.pack(0, schema_version)


def deserialize_confluent_wire_header(raw):
    """Parses the byte prefix for Confluent Wire Format-style Kafka messages.

    Parameters
    ----------
    raw : `bytes`
        The 5-byte encoded message prefix.

    Returns
    -------
    schema_version : `int`
        A version number which indicates the Confluent Schema Registry ID
        number of the Avro schema used to encode the message that follows this
        header.
    """
    _, version = _ConfluentWireFormatHeader.unpack(raw)
    return version


def serialize_alert(alert, schema=latest_schema, schema_id=0):
    """Serialize an alert to a byte sequence for sending to Kafka.

    Parameters
    ----------
    alert : `dict`
        An alert payload to be serialized.
    schema : `dict`, optional
        An Avro schema definition describing how to encode `alert`. By default,
        the latest schema is used.
    schema_id : `int`, optional
        The Confluent Schema Registry ID of the schema. By default, 0 (an
        invalid ID) is used, indicating that the schema is not registered.

    Returns
    -------
    serialized : `bytes`
        The byte sequence describing the alert, including the Confluent Wire
        Format prefix.
    """
    buf = io.BytesIO()
    # TODO: Use a proper schema versioning system
    buf.write(serialize_confluent_wire_header(schema_id))
    fastavro.schemaless_writer(buf, schema, alert)
    return buf.getvalue()


def deserialize_alert(alert_bytes, schema=latest_schema):
    """Deserialize an alert message from Kafka.

    Paramaters
    ----------
    alert_bytes : `bytes`
        Binary-encoding serialized Avro alert, including Confluent Wire
        Format prefix.
    schema : `dict`, optional
        An Avro schema definition describing how to encode `alert`. By default,
        the latest schema is used.

    Returns
    -------
    alert : `dict`
        An alert payload.
    """
    header_bytes = alert_bytes[:5]
    version = deserialize_confluent_wire_header(header_bytes)
    assert version == 0
    content_bytes = io.BytesIO(alert_bytes[5:])
    return fastavro.schemaless_reader(content_bytes, schema)



# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ['AlertFilter', 'Exporter', 'StreamExporter']


class AlertFilter(object):

    def __init__(self, outputStream):
        self.outputStream = outputStream
        self.visitId = 'visitId'

    def __call__(self, schema, alert):
        visit = self.getVisit(alert)
        if visit != self.visitId:
            self.alertCount = 1
            self.visitId = visit
        if self.filter(alert) and self.alertCount <= 20:
            self.alertCount += 1
            self.outputStream(schema, alert)

    def getVisit(self, alert):
        ccdVisitId = alert['diaSource']['ccdVisitId']
        # TODO Visit ID scraping will depend on format of ccdVisitId
        visit = str(ccdVisitId)[-5:]
        return visit

    def filter(self, alert):
        raise NotImplementedError


class Exporter(object):

    def __call__(self, schema, alert):
        self.export(schema, alert)

    def export(self, schema, alert):
        raise NotImplementedError


class StreamExporter(Exporter):

    def __init__(self, producer):
        self.producer = producer

    def export(self, schema, alert):
        self.producer.send(schema, alert)



# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from .filterBase import AlertFilter

class Filter001(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):  # 20th mag
            return True
        else:
            return False


class Filter002(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter003(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter004(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter005(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter006(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter007(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter008(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter009(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter010(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter011(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter012(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter013(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter014(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter015(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter016(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter017(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter018(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter019(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter020(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter021(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter022(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter023(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter024(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter025(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter026(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter027(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter028(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter029(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter030(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter031(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter032(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter033(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter034(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter035(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter036(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter037(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter038(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter039(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter040(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter041(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter042(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter043(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter044(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter045(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter046(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter047(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter048(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter049(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter050(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter051(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter052(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter053(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter054(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter055(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter056(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter057(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter058(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter059(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter060(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter061(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter062(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter063(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter064(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter065(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter066(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter067(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter068(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter069(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter070(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter071(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter072(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter073(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter074(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter075(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter076(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter077(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter078(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter079(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter080(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False



class Filter081(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter082(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter083(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter084(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter085(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter086(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter087(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter088(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter089(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter090(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter091(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter092(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter093(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter094(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter095(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter096(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter097(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter098(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter099(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False


class Filter100(AlertFilter):
    def filter(self, alert):
        if ((alert['diaSource']['snr'] > 5) &
           (alert['diaSource']['diffFlux'] > 0.00003631)):
            return True
        else:
            return False



#!/usr/bin/env python
#
# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Consumes stream for printing all messages to the console.

Note that consumers with the same group ID share a stream.
To run multiple consumers each printing all messages,
each consumer needs a different group.
"""

import argparse
import sys
import os
from lsst.alert.stream import alertConsumer


def msg_text(message):
    """Remove postage stamp cutouts from an alert message.
    """
    message_text = {k: message[k] for k in message
                    if k not in ['cutoutDifference', 'cutoutTemplate']}
    return message_text


def write_stamp_file(stamp_dict, output_dir):
    """Given a stamp dict that follows the cutout schema,
       write data to a file in a given directory.
    """
    try:
        filename = stamp_dict['fileName']
        try:
            os.makedirs(output_dir)
        except OSError:
            pass
        out_path = os.path.join(output_dir, filename)
        with open(out_path, 'wb') as f:
            f.write(stamp_dict['stampData'])
    except TypeError:
        sys.stderr.write('%% Cannot get stamp\n')
    return


def alert_filter(alert, stampdir=None):
    """Filter to apply to each alert.
       See schemas here: https://github.com/lsst-dm/sample-avro-alert
    """
    data = msg_text(alert)
    if data:  # Write your condition statement here
        print(data)  # Print main alert data to screen
        if stampdir:  # Collect all postage stamps **There are no stamps**
            write_stamp_file(
                alert.get('cutoutDifference'), stampdir)
            write_stamp_file(
                alert.get('cutoutTemplate'), stampdir)
    return


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('broker', type=str,
                        help='Hostname or IP and port of Kafka broker.')
    parser.add_argument('topic', type=str,
                        help='Name of Kafka topic to listen to.')
    parser.add_argument('interval', type=int,
                        help='Print every Nth alert.')
    parser.add_argument('--group', type=str,
                        help='Globally unique name of the consumer group. '
                        'Consumers in the same group will share messages '
                        '(i.e., only one consumer will receive a message, '
                        'as in a queue). Default is value of $HOSTNAME.')
    parser.add_argument('--stampDir', type=str,
                        help='Output directory for writing postage stamp'
                        'cutout files. **THERE ARE NO STAMPS RIGHT NOW.**')

    args = parser.parse_args()

    # Configure consumer connection to Kafka broker
    conf = {'bootstrap.servers': args.broker,
            'default.topic.config': {'auto.offset.reset': 'smallest'}}
    if args.group:
        conf['group.id'] = args.group
    else:
        conf['group.id'] = os.environ['HOSTNAME']

    # Start consumer and print alert stream
    with alertConsumer.AlertConsumer(args.topic, **conf) as streamReader:
        msg_count = 0
        while True:
            try:
                schema, msg = streamReader.poll()

                if msg is None:
                    continue
                else:
                    msg_count += 1
                    if msg_count % args.interval == 0:
                        # Apply filter to each alert
                        alert_filter(msg, args.stampDir)

            except alertConsumer.EopError as e:
                # Write when reaching end of partition
                sys.stderr.write(e.message)
            except IndexError:
                sys.stderr.write('%% Data cannot be decoded\n')
            except UnicodeDecodeError:
                sys.stderr.write('%% Unexpected data format received\n')
            except KeyboardInterrupt:
                sys.stderr.write('%% Aborted by user\n')
                sys.exit()



#!/usr/bin/env python
#
# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""Consumes stream for monitoring.

Note that consumers with the same group ID share a stream.
To run multiple consumers, each consumer needs a different group.
"""

import argparse
import os
import sys
from lsst.alert.stream import alertConsumer


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('broker', type=str,
                        help='Hostname or IP and port of Kafka broker.')
    parser.add_argument('topic', type=str,
                        help='Name of Kafka topic to listen to.')
    parser.add_argument('--group', type=str,
                        help='Globally unique name of the consumer group. '
                        'Consumers in the same group will share messages '
                        '(i.e., only one consumer will receive a message, '
                        'as in a queue). Default is value of $HOSTNAME.')
    args = parser.parse_args()

    # Configure consumer connection to Kafka broker
    conf = {'bootstrap.servers': args.broker,
            'default.topic.config': {'auto.offset.reset': 'smallest'}}
    if args.group:
        conf['group.id'] = args.group
    else:
        conf['group.id'] = os.environ['HOSTNAME']

    # Start consumer and monitor alert stream
    with alertConsumer.AlertConsumer(args.topic, **conf) as streamWatcher:

        while True:
            try:
                schema, msg = streamWatcher.poll()
                if msg is not None:
                    print(msg)
            except alertConsumer.EopError as e:
                # Write when reaching end of partition
                sys.stderr.write(e.message)
            except KeyboardInterrupt:
                sys.stderr.write('%% Aborted by user\n')
                sys.exit()



# This file is part of alert_stream.
#
# Developed for the LSST Data Management System.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import struct
from io import BytesIO

import confluent_kafka

from lsst.alert.packet import SchemaRegistry

__all__ = ['AlertProducer']


class AlertProducer(object):
    """Alert stream producer with Kafka.

    Parameters
    ----------
    topic : `str`
        The name of the topic stream for writing.
    **kwargs
        Keyword arguments for configuring confluent_kafka.Producer().
    """

    def __init__(self, topic, **kwargs):
        self.producer = confluent_kafka.Producer(**kwargs)
        self.topic = topic

    def send(self, schema, data):
        """Sends a message to Kafka stream.

        The message is encoded following the `Confluent Wire Format`_. Thus:

        Byte 0:     ``0``.
        Byte 1-4:   A 4-byte schema ID.
        Byte 5-...: The packet data, Avro encoded following `schema`.

        .. _Confluent Wire Format: https://docs.confluent.io/current/schema-registry/docs/serializer-formatter.html#wire-format

        Parameters
        ----------
        schema : `lsst.alert.packet.Schema`
            The Avro schema for encoding data. Although one is not required to
            construct the producer, you can't begin sending data until one has
            been supplied.
        data : `dict`
            Message content. Must comply with the `schema`.
        """
        outgoing_bytes = BytesIO()
        outgoing_bytes.write(struct.pack("!b", 0))
        outgoing_bytes.write(struct.pack("!I", SchemaRegistry.calculate_id(schema)))
        outgoing_bytes.write(schema.serialize(data))
        self.producer.produce(self.topic, outgoing_bytes.getvalue())

    def flush(self):
        return self.producer.flush()
